// alumno.cpp - Definicin de los mtodos de la clase CAlumno
#include <iostream>
#include "alumno.h"
using namespace std;

CAlumno::CAlumno(long dni, string nom, string dir):
DNI(dni), nombre(nom), direccion(dir)
{
  asignarDNI(dni);
  asignatura.reserve(10); // espacio para 10 objetos CAsignatura
}

CAlumno::~CAlumno()
{
  //cout << "destructor CAlumno\n";
  for (unsigned int i = 0; i < numeroAsignaturas(); i++)
    delete asignatura[i];
}

CAlumno::CAlumno(const CAlumno& x)
{
  *this = x;
}

CAlumno& CAlumno::operator=(const CAlumno& x)
{
  // Eliminar las asignaturas del objeto CAlumno destino (*this)
  if (asignatura.size())
  {
    for (unsigned int i = 0; i < asignatura.size(); i++)
      delete asignatura[i];
    // Eliminar todos los elementos de la matriz asignatura
    asignatura.clear();
  }

  // Copiar el alumno origen, x, en el alumno destino
  DNI = x.DNI;
  nombre = x.nombre;
  direccion = x.direccion;
  for (unsigned int i = 0; i < x.asignatura.size(); i++)
    asignatura.push_back(x.asignatura[i]->clonar());

  return *this;
}

void CAlumno::asignarDNI(long dni)
{
  if (dni < 0)
  {
    throw "error: el DNI no puede ser negativo\n";
    dni = 0;
  }
  DNI = dni;
}

long CAlumno::obtenerDNI()
{
  return DNI;
}

void CAlumno::asignarNombre(string& nom)
{
  nombre = nom;
}

string CAlumno::obtenerNombre()
{
  return nombre;
}

void CAlumno::asignarDireccion(string& dir)
{
  direccion = dir;
}

string CAlumno::obtenerDireccion()
{
  return direccion;
}

CAsignatura *CAlumno::obtenerAsignatura(unsigned int i)
{
  if (numeroAsignaturas() == 0) return 0;
  if (i >= 0 && i < numeroAsignaturas())
    return asignatura[i];
  else
  {
    cerr << "error: ndice fuera de lmites\n";
    return 0;
  }
}

void CAlumno::anyadirAsignatura(CAsignatura *asig)
{
  asignatura.push_back(asig->clonar());
}

bool CAlumno::estaEnActa(int id, int& i)
{
  // En i se devuelve la posicin de la asignatura id en la lista
  if (numeroAsignaturas() == 0) return false;
  // Un alumno pertenece al acta de la asignatura id si se ha
  // matriculado, an no ha aprobado y no excede el nmero
  // de convocatorias.
  for (i = 0; i < numeroAsignaturas(); i++)
  {
    CAsignatura& asig = *obtenerAsignatura(i);
    if (asig.obtenerID() != id) continue;
    if (asig) // si la asignatura est aprobada ...
      return false;
    if (asig.convocatoriasConsumidas() == 6)
    {
      cerr << "error: convocatorias agotadas\n";
      break;
    }
    return true; // est en el acta.
  }
  return false;  // no est en el acta.
}

size_t CAlumno::numeroAsignaturas()
{
  return asignatura.size();
}

CAlumno& CAlumno::operator<<(CAsignatura *asig)
{
  asignatura.push_back(asig->clonar());
  return *this;
}

