// subindices.h
#if !defined( _SUBINDICES_H_ )
#define _SUBINDICES_H_

//////////////////////////////////////////////////////////////////
// Clase para acumular los subndices. Bsicamente crea una lista
// con los subndices de un elemento determinado, [][][]
class CSubindices
{
    int *m_pSubindices;   // matriz para los subndices
    int m_nSubindices;    // nmero total de subndices
    int *m_pSiguienteInd; // localizacin del siguiente subndice
    int m_nSubindsActual; // subndices que hay actualmente
  public:
    CSubindices(int nTotalSubinds, int PrimerSubind);
    ~CSubindices() { delete [] m_pSubindices; }
    int nSubindsActual() { return m_nSubindsActual; }
    void AnyadirSigSubind(int subind);
    int Desplazamiento(CDimensiones &dims) const;
};

CSubindices::CSubindices(int nTotalSubinds, int PrimerSubind)
{
  if (nTotalSubinds < 1)
  {
    cerr << "Nmero de subndices incorrecto\n";
    nTotalSubinds = 1;
    PrimerSubind = 0;
  }
  m_pSubindices = m_pSiguienteInd = new int[m_nSubindices = nTotalSubinds];
  *m_pSiguienteInd++ = PrimerSubind;
  m_nSubindsActual = 1;
}

// Aadir el siguiente subndice de un elemento a su matriz
// de subndices
void CSubindices::AnyadirSigSubind(int subind)
{
  if (m_nSubindsActual < m_nSubindices)
  {
    *m_pSiguienteInd++ = subind;
    m_nSubindsActual++;
  }
}

// Calcular el desplazamiento en el espacio fsico de la matriz,
// correspondiente a los subndices del elemento elegido.
// Fsicamente es una matriz lineal.
int CSubindices::Desplazamiento(CDimensiones &dims) const
{
  int desplazamiento = 0; // resultado
  int *pSubind = m_pSubindices; // subndices del elemento accedido
  const int *dimension = dims.DimensionesMatriz(); // dimensiones
  int nDims = dims.nDimensiones();

  while (nDims--)
  {
    // Verificar si los subndices estn dentro del rango
    if (*pSubind < 0 || *pSubind >= *dimension)
    {
      cerr << "Subndice fuera de rango\n";
      return -1;
    }
    desplazamiento += *pSubind++;
    if (nDims) desplazamiento *= *++dimension;
  }
  return desplazamiento;
}
//////////////////////////////////////////////////////////////////

#endif // _SUBINDICES_H_

